// (C) 2023-2025 GoodData Corporation
import { put, call, takeLatest, select, cancelled } from "redux-saga/effects";
import difference from "lodash/difference.js";
import { getAttributeFilterContext } from "../common/sagas.js";
import { selectElementsForm, selectWithoutApply } from "../common/selectors.js";
import { elementsSaga } from "../elements/elementsSaga.js";
import { selectLoadElementsOptions } from "../elements/elementsSelectors.js";
import { actions } from "../store/slice.js";
import { selectCommittedSelection, selectWorkingSelection } from "../store/selectors.js";
import { shouldExcludePrimaryLabel } from "../utils.js";
/**
 * @internal
 */
export function* loadIrrelevantElementsWorker() {
    yield takeLatest([actions.loadIrrelevantElementsRequest.match, actions.loadIrrelevantElementsCancelRequest.match], loadIrrelevantElementsSaga);
}
/**
 * @internal
 */
export function* loadIrrelevantElementsSaga(action) {
    if (actions.loadIrrelevantElementsCancelRequest.match(action)) {
        // Saga was triggered by cancel request - do nothing, finally statement was already called, because takeLatest can run only one saga at a time === the previous one was canceled
        return;
    }
    const context = yield call(getAttributeFilterContext);
    const abortController = new AbortController();
    const { payload: { correlation }, } = action;
    try {
        yield put(actions.loadIrrelevantElementsStart({ correlation }));
        const elementsForm = yield select(selectElementsForm);
        const loadOptions = yield select(selectLoadElementsOptions);
        const withoutApply = yield select(selectWithoutApply);
        const allSelectedElementTitles = yield select(withoutApply ? selectWorkingSelection : selectCommittedSelection);
        const elements = elementsForm === "values"
            ? { values: allSelectedElementTitles }
            : { uris: allSelectedElementTitles };
        const loadOptionsWithElements = {
            ...loadOptions,
            signal: abortController.signal,
            excludePrimaryLabel: shouldExcludePrimaryLabel(context, elementsForm),
            search: "", // search is not relevant here
            elements,
        };
        const relevantElementsResult = yield call(elementsSaga, loadOptionsWithElements);
        const relevantElementTitles = relevantElementsResult.elements.map((elem) => context.enableDuplicatedLabelValuesInAttributeFilter ? elem.uri : elem.title);
        const irrelevantSelectionTitles = difference(allSelectedElementTitles, relevantElementTitles);
        yield put(actions.loadIrrelevantElementsSuccess({ elementTitles: irrelevantSelectionTitles, correlation }));
    }
    catch (error) {
        yield put(actions.loadIrrelevantElementsError({
            error,
            correlation,
        }));
    }
    finally {
        if (yield cancelled()) {
            abortController.abort();
            yield put(actions.loadIrrelevantElementsCancel({ correlation }));
        }
    }
}
//# sourceMappingURL=loadIrrelevantElementsSaga.js.map