import React, { ComponentType } from "react";
import { DateFilterGranularity, WeekStart } from "@gooddata/sdk-model";
import { DateFilterOption, IDateFilterOptionsByType } from "./interfaces/index.js";
import { IFilterButtonCustomIcon, VisibilityMode } from "../shared/index.js";
import { IFilterConfigurationProps } from "./DateFilterBody/types.js";
import { IDateFilterButtonProps } from "./DateFilterButton/DateFilterButton.js";
import { OverlayPositionType } from "@gooddata/sdk-ui-kit";
/**
 * Props of the {@link DateFilter} component that are reflected in the state.
 *
 * @public
 */
export interface IDateFilterStatePropsIntersection {
    excludeCurrentPeriod: boolean;
    selectedFilterOption: DateFilterOption;
}
/**
 * Props of the {@link DateFilter} component.
 * @public
 */
export interface IDateFilterOwnProps extends IDateFilterStatePropsIntersection {
    filterOptions: IDateFilterOptionsByType;
    availableGranularities: DateFilterGranularity[];
    isEditMode?: boolean;
    openOnInit?: boolean;
    customFilterName?: string;
    dateFilterMode: VisibilityMode;
    dateFormat?: string;
    locale?: string;
    isTimeForAbsoluteRangeEnabled?: boolean;
    showDropDownHeaderMessage?: boolean;
    weekStart?: WeekStart;
    /**
     * Represents a custom icon with associated tooltip information.
     *
     * @alpha
     */
    customIcon?: IFilterButtonCustomIcon;
    /**
     * Represents a custom component for configuration.
     *
     * @alpha
     */
    FilterConfigurationComponent?: React.ComponentType<IFilterConfigurationProps>;
    /**
     * This enables filter mode without apply button.
     * If true, it is responsibility of a client, to appy filters when needed.
     * Typically uses onSelect callback to catch filter state.
     * Note, onApply callback is not called when this is true.
     *
     * @alpha
     */
    withoutApply?: boolean;
    /**
     * Working filter option used for synchronization inner filter state with outer given state.
     * Makes a controlled component state out of this.
     *
     * @alpha
     * @deprecated dont use. Will be removed in future releases.
     */
    workingSelectedFilterOption?: DateFilterOption;
    /**
     * Working filter exclude current period used for synchronization inner filter state with outer given state.
     * Makes a controlled component state out of this.
     *
     * @alpha
     * @deprecated dont use. Will be removed in future releases.
     */
    workingExcludeCurrentPeriod?: boolean;
    /**
     * If new apply all filters at once mode is enabled
     */
    enableDashboardFiltersApplyModes?: boolean;
    /**
     * Specify custom button component
     *
     * @alpha
     */
    ButtonComponent?: ComponentType<IDateFilterButtonProps>;
    /**
     * Specifies the overlay position type for the date filter dropdown.
     */
    overlayPositionType?: OverlayPositionType;
}
/**
 * Callback props of the {@link DateFilter} component.
 *
 * @public
 */
export interface IDateFilterCallbackProps {
    onApply: (dateFilterOption: DateFilterOption, excludeCurrentPeriod: boolean) => void;
    onSelect?: (dateFilterOption: DateFilterOption, excludeCurrentPeriod: boolean) => void;
    onCancel?: () => void;
    onOpen?: () => void;
    onClose?: () => void;
}
/**
 * All the props of the {@link DateFilter} component.
 *
 * @public
 */
export interface IDateFilterProps extends IDateFilterOwnProps, IDateFilterCallbackProps {
}
/**
 * State of the {@link DateFilter} component.
 *
 * @public
 */
export interface IDateFilterState extends IDateFilterStatePropsIntersection {
    initExcludeCurrentPeriod: boolean;
    initSelectedFilterOption: DateFilterOption;
    initWorkingExcludeCurrentPeriod: boolean;
    initWorkingSelectedFilterOption: DateFilterOption;
    isExcludeCurrentPeriodEnabled: boolean;
}
/**
 * {@link https://sdk.gooddata.com/gooddata-ui/docs/date_filter_component.html | DateFilter} is a component for configuring a date filter value.
 *
 * @public
 */
export declare class DateFilter extends React.PureComponent<IDateFilterProps, IDateFilterState> {
    static defaultProps: Partial<IDateFilterProps>;
    static getDerivedStateFromProps(nextProps: IDateFilterProps, prevState: IDateFilterState): IDateFilterState;
    private static getStateFromProps;
    private static getStateFromWorkingProps;
    private static getStateFromSelectedOption;
    private static checkInitialFilterOption;
    constructor(props: IDateFilterProps);
    componentDidMount(): void;
    render(): React.JSX.Element;
    private handleApplyClick;
    private onChangesDiscarded;
    private onCancelClicked;
    private onDropdownOpenChanged;
    private handleExcludeCurrentPeriodChange;
    private handleSelectedFilterOptionChange;
    private handleSelectChange;
}
//# sourceMappingURL=DateFilter.d.ts.map