// (C) 2007-2025 GoodData Corporation
import React, { useCallback, useMemo, useRef, useState } from "react";
import flow from "lodash/flow.js";
import noop from "lodash/noop.js";
import { MediaQuery } from "react-responsive";
import format from "date-fns/format/index.js";
import { Dropdown } from "@gooddata/sdk-ui-kit";
import { IntlWrapper } from "@gooddata/sdk-ui";
import { MediaQueries } from "../constants/index.js";
import { DateFilterButtonLocalized } from "./DateFilterButtonLocalized/DateFilterButtonLocalized.js";
import { DateFilterBody } from "./DateFilterBody/DateFilterBody.js";
import { applyExcludeCurrentPeriod } from "./utils/PeriodExclusion.js";
import { DEFAULT_DATE_FORMAT, TIME_FORMAT_WITH_SEPARATOR } from "./constants/Platform.js";
import { filterVisibleDateFilterOptions, sanitizePresetIntervals } from "./utils/OptionUtils.js";
import isEmpty from "lodash/isEmpty.js";
import { createDateFilterKeyboardHandler } from "./accessibility/keyboardNavigation.js";
import { DATE_FILTER_SELECTED_LIST_ITEM_ID } from "./accessibility/elementId.js";
export const verifyDateFormat = (dateFormat) => {
    try {
        // Try to format the current date to verify if dateFormat is a valid format.
        format(new Date(), dateFormat);
        return dateFormat;
    }
    catch {
        // If an error occurs, then dateFormat is invalid and the default format should be used instead. Also, a warning is written in the console.
        console.warn(`Unsupported date format ${dateFormat}, the default format ${DEFAULT_DATE_FORMAT} is used instead.`);
        return DEFAULT_DATE_FORMAT;
    }
};
const adjustDateFormatForDisplay = (dateFormat, isTimeForAbsoluteRangeEnabled = false) => isTimeForAbsoluteRangeEnabled ? dateFormat + TIME_FORMAT_WITH_SEPARATOR : dateFormat;
export const DateFilterCore = ({ originalSelectedFilterOption, originalExcludeCurrentPeriod, selectedFilterOption, excludeCurrentPeriod, onDropdownOpenChanged, customFilterName, dateFormat, disabled, openOnInit, locale, filterOptions, isTimeForAbsoluteRangeEnabled, weekStart, customIcon, FilterConfigurationComponent, onApplyClick, onCancelClick, showDropDownHeaderMessage, withoutApply, ButtonComponent, overlayPositionType, ...dropdownBodyProps }) => {
    const [isConfigurationOpen, setIsConfigurationOpen] = useState(false);
    const verifiedDateFormat = verifyDateFormat(dateFormat);
    const dateFilterContainerRef = useRef(null);
    const dateFilterBodyRef = useRef(null);
    const filteredFilterOptions = useMemo(() => {
        return flow(filterVisibleDateFilterOptions, sanitizePresetIntervals)(filterOptions);
    }, [filterOptions]);
    const closeConfiguration = () => {
        setIsConfigurationOpen(false);
    };
    const openConfiguration = () => {
        setIsConfigurationOpen(true);
    };
    const cancelConfiguration = () => {
        closeConfiguration();
        onCancelClick();
    };
    const handleOpenStateChanged = (isOpen) => {
        onDropdownOpenChanged(isOpen);
        if (!isOpen) {
            closeConfiguration();
        }
    };
    const lastValidSelectedFilterOption = useLastValidValue(selectedFilterOption, isEmpty(dropdownBodyProps.errors));
    const lastValidExcludeCurrentPeriod = useLastValidValue(excludeCurrentPeriod, isEmpty(dropdownBodyProps.errors));
    const handleKeyDown = useCallback((event, closeDropdown = noop) => {
        const keyboardHandler = createDateFilterKeyboardHandler({
            dateFilterContainerRef,
            dateFilterBodyRef,
            closeDropdown,
        });
        keyboardHandler(event);
    }, [dateFilterContainerRef, dateFilterBodyRef]);
    return (React.createElement(IntlWrapper, { locale: locale || "en-US" },
        React.createElement(MediaQuery, { query: MediaQueries.IS_MOBILE_DEVICE }, (isMobile) => {
            const dateFilterButton = (isOpen = false, buttonRef = null, dropdownId = "", toggleDropdown = noop) => (React.createElement(DateFilterButtonLocalized, { disabled: disabled, isMobile: isMobile, isOpen: isOpen, dateFilterOption: applyExcludeCurrentPeriod(withoutApply ? lastValidSelectedFilterOption : originalSelectedFilterOption, withoutApply ? lastValidExcludeCurrentPeriod : originalExcludeCurrentPeriod), dateFormat: adjustDateFormatForDisplay(verifiedDateFormat, isTimeForAbsoluteRangeEnabled), customFilterName: customFilterName, customIcon: customIcon, onClick: toggleDropdown, ButtonComponent: ButtonComponent, buttonRef: buttonRef, dropdownId: dropdownId }));
            return (React.createElement(Dropdown, { openOnInit: openOnInit, closeOnParentScroll: true, closeOnMouseDrag: true, closeOnOutsideClick: true, enableEventPropagation: true, autofocusOnOpen: true, initialFocus: DATE_FILTER_SELECTED_LIST_ITEM_ID, alignPoints: [
                    { align: "bl tl" },
                    { align: "tr tl" },
                    { align: "br tr", offset: { x: -11 } },
                    { align: "tr tl", offset: { x: 0, y: -100 } },
                    { align: "tr tl", offset: { x: 0, y: -50 } },
                ], onOpenStateChanged: handleOpenStateChanged, overlayPositionType: overlayPositionType, renderButton: ({ isOpen, buttonRef, dropdownId, toggleDropdown }) => (React.createElement("span", { onClick: disabled ? noop : toggleDropdown }, dateFilterButton(isOpen, buttonRef, dropdownId, toggleDropdown))), ignoreClicksOnByClass: [
                    ".s-do-not-close-dropdown-on-click",
                    ".rdp-day", // absolute range picker calendar items
                ], renderBody: ({ closeDropdown, ariaAttributes }) => {
                    return isConfigurationOpen ? (React.createElement(FilterConfigurationComponent, { onSaveButtonClick: closeConfiguration, onCancelButtonClick: cancelConfiguration })) : (
                    // Dropdown component uses React.Children.map and adds special props to this component
                    // https://stackoverflow.com/questions/32370994/how-to-pass-props-to-this-props-children
                    React.createElement("div", { role: "dialog", className: "gd-extended-date-filter-container", id: ariaAttributes.id, ref: dateFilterContainerRef, onKeyDown: (e) => handleKeyDown(e, closeDropdown) },
                        React.createElement(DateFilterBody, { ...dropdownBodyProps, selectedFilterOption: selectedFilterOption, excludeCurrentPeriod: excludeCurrentPeriod, showHeaderMessage: showDropDownHeaderMessage, onApplyClick: onApplyClick, onCancelClick: onCancelClick, filterOptions: filteredFilterOptions, isMobile: isMobile, closeDropdown: closeDropdown, dateFilterButton: dateFilterButton(), dateFormat: verifiedDateFormat, isTimeForAbsoluteRangeEnabled: isTimeForAbsoluteRangeEnabled, weekStart: weekStart, isConfigurationEnabled: FilterConfigurationComponent !== undefined, onConfigurationClick: openConfiguration, withoutApply: withoutApply, ref: dateFilterBodyRef })));
                } }));
        })));
};
function useLastValidValue(value, isValid) {
    const lastValidValue = useRef();
    if (isValid) {
        lastValidValue.current = value;
    }
    return lastValidValue.current;
}
//# sourceMappingURL=DateFilterCore.js.map