// (C) 2025 GoodData Corporation
import React, { useCallback, useState, useEffect } from "react";
import cx from "classnames";
import isEmpty from "lodash/isEmpty.js";
import { useId, isEnterKey, isArrowKey } from "@gooddata/sdk-ui-kit";
import { convertPlatformDateStringToDate } from "../utils/DateConversions.js";
import { DateRangePickerInputFieldBody } from "./DateRangePickerInputFieldBody.js";
import { buildAriaDescribedByValue, parseDate, formatDate, isValidDate, getPlatformStringFromDate, } from "./utils.js";
import { InputErrorMessage } from "./InputErrorMessage.js";
const getErrorMessage = (errorCause, isDateOrderError, messages) => {
    if (errorCause === "empty") {
        return messages.emptyDate;
    }
    else if (errorCause === "invalid") {
        return messages.invalidDate;
    }
    else if (isDateOrderError) {
        return messages.startDateAfterEndDate;
    }
    return undefined;
};
const useDateInput = ({ value, dateFormat, onChange, onKeyDown, isDateOrderError = false, errorMessageTexts, }) => {
    const [inputValue, setInputValue] = useState(formatDate(value, dateFormat));
    const [errorCause, setErrorCause] = useState(undefined);
    const hasError = errorCause !== undefined || isDateOrderError;
    const errorMessage = getErrorMessage(errorCause, isDateOrderError, errorMessageTexts);
    useEffect(() => {
        const newValue = formatDate(value, dateFormat);
        setInputValue(newValue);
        if (newValue !== undefined) {
            setErrorCause(undefined);
        }
    }, [value, dateFormat]);
    const onDateInputChange = useCallback((event) => {
        const newValue = event.target.value;
        setInputValue(newValue);
        const newDate = parseDate(newValue, dateFormat);
        // commit changed value when time is valid but only once, then it is reported again only on blur
        if (isValidDate(newDate) && hasError) {
            setErrorCause(undefined);
            onChange(newDate);
        }
    }, [dateFormat, onChange, hasError]);
    const onSubmitValue = useCallback((shouldSubmitForm) => {
        if (isEmpty(inputValue)) {
            setErrorCause("empty");
            onChange(undefined, shouldSubmitForm);
            return;
        }
        const date = parseDate(inputValue, dateFormat);
        if (date === undefined) {
            setErrorCause("invalid");
            onChange(undefined, shouldSubmitForm);
            return;
        }
        setErrorCause(null);
        onChange(date, shouldSubmitForm);
    }, [dateFormat, inputValue, onChange]);
    const onDateInputBlur = useCallback(() => onSubmitValue(false), [onSubmitValue]);
    const onMobileDateChange = useCallback((event) => {
        const value = event.target.value;
        const selectedDate = convertPlatformDateStringToDate(value);
        const isValid = isValidDate(selectedDate);
        onChange(isValid ? selectedDate : undefined);
        setErrorCause(isValid ? undefined : "empty");
    }, [onChange]);
    const onDateInputKeyDown = useCallback((e) => {
        if (isEnterKey(e)) {
            onSubmitValue(true);
        }
        else if (isArrowKey(e)) {
            e.stopPropagation(); // allow navigation in the input
        }
        else {
            onKeyDown(e);
        }
    }, [onSubmitValue, onKeyDown]);
    return {
        inputValue,
        errorMessage,
        onDateInputChange,
        onMobileDateChange,
        onDateInputBlur,
        onDateInputKeyDown,
    };
};
export const DateInput = React.forwardRef(({ value, inputLabel, onKeyDown, onClick, onChange, dateFormat, accessibilityConfig, isMobile, isTimeEnabled, isDateOrderError, errorMessageTexts, }, ref) => {
    const { inputValue, errorMessage, onDateInputChange, onDateInputKeyDown, onMobileDateChange, onDateInputBlur, } = useDateInput({
        value,
        dateFormat,
        onChange,
        onKeyDown,
        isDateOrderError,
        errorMessageTexts,
    });
    const inputLabelId = useId();
    const inputErrorId = useId();
    const ariaProps = {
        "aria-label": accessibilityConfig.ariaLabel,
        "aria-labelledby": inputLabelId,
        "aria-describedby": buildAriaDescribedByValue([
            accessibilityConfig.inputHintId,
            errorMessage ? inputErrorId : undefined,
        ]),
        ...(errorMessage ? { "aria-invalid": true } : {}),
    };
    const MobileInput = (React.createElement(DateRangePickerInputFieldBody, { type: "date", className: cx("s-date-range-picker-date", "gd-date-range-picker-input", "gd-date-range-picker-input-native", {
            "gd-date-range-picker-input-error": !!errorMessage,
            "has-error": !!errorMessage,
        }), onChange: onMobileDateChange, value: getPlatformStringFromDate(value), ...ariaProps }));
    const DesktopInput = (React.createElement("div", { className: cx("gd-date-range-picker-input", {
            "gd-date-range-picker-input-error": !!errorMessage,
            "has-error": !!errorMessage,
        }) },
        React.createElement("span", { className: "gd-icon-calendar" }),
        React.createElement("input", { ref: ref, placeholder: dateFormat, onChange: onDateInputChange, onClick: onClick, onBlur: onDateInputBlur, onKeyDown: onDateInputKeyDown, value: inputValue, className: "input-text s-date-range-picker-input-field", ...ariaProps })));
    return (React.createElement("div", { className: cx("gd-date-range-column", { "gd-date-range-column--with-time": isTimeEnabled }) },
        React.createElement("label", { id: inputLabelId }, inputLabel),
        isMobile ? MobileInput : DesktopInput,
        React.createElement(InputErrorMessage, { descriptionId: inputErrorId, errorText: errorMessage })));
});
DateInput.displayName = "DateInput";
//# sourceMappingURL=DateInput.js.map