// (C) 2007-2025 GoodData Corporation
import React, { useState, useRef, useEffect, useCallback } from "react";
import { injectIntl } from "react-intl";
import { StartDateInputField } from "./StartDateInputField.js";
import { EndDateInputField } from "./EndDateInputField.js";
import { DateRangeHint } from "./DateRangeHint.js";
import { DayPicker } from "./DatePicker.js";
import { DATE_INPUT_HINT_ID, TIME_INPUT_HINT_ID } from "./types.js";
import { isValidDate, setTimeToDate, getTimeFromDate } from "./utils.js";
const isClickOutsideOfCalendar = (event, container, startDateInput, endDateInput) => {
    return (container &&
        !container.contains(event.target) &&
        startDateInput &&
        !startDateInput.contains(event.target) &&
        endDateInput &&
        !endDateInput.contains(event.target));
};
const useCalendarPopup = () => {
    const [isOpen, setIsOpen] = useState(false);
    const [selectedInput, setSelectedInput] = useState();
    const startDateInputRef = useRef(null);
    const endDateInputRef = useRef(null);
    const calendarPopupRef = useRef(null);
    const onStartDateClick = useCallback(() => {
        setSelectedInput("from");
        setIsOpen(true);
    }, []);
    const onEndDateClick = useCallback(() => {
        setSelectedInput("to");
        setIsOpen(true);
    }, []);
    const onDateInputKeyDown = useCallback((e) => {
        if (isOpen && (e.key === "Escape" || e.key === "Tab")) {
            setIsOpen(false);
            e.stopPropagation(); // prevent closing of the dropdown when just the calendar should close
        }
    }, [isOpen]);
    const closeCalendarPopup = useCallback(() => setIsOpen(false), []);
    const handleClickOutside = useCallback((event) => {
        if (isClickOutsideOfCalendar(event, calendarPopupRef.current, startDateInputRef.current, endDateInputRef.current)) {
            setIsOpen(false);
        }
    }, []);
    useEffect(() => {
        document.addEventListener("mousedown", handleClickOutside);
        return () => document.removeEventListener("mousedown", handleClickOutside);
    }, [handleClickOutside]);
    return {
        selectedInput,
        isCalendarPopupOpen: isOpen,
        onDateInputKeyDown,
        onStartDateClick,
        onEndDateClick,
        calendarPopupRef,
        startDateInputRef,
        endDateInputRef,
        closeCalendarPopup,
    };
};
const isSameDay = (d1, d2) => {
    return (!!d1 &&
        !!d2 &&
        d1.getFullYear() === d2.getFullYear() &&
        d1.getMonth() === d2.getMonth() &&
        d1.getDate() === d2.getDate());
};
const setStartAfterEndDateError = (fieldDateOrderErrorSetter, otherDateOrderFieldSetter, start, end) => {
    const isValidStartDate = isValidDate(start);
    const isValidEndDate = isValidDate(end);
    const isStartBeforeEnd = start <= end;
    if (!isValidStartDate || !isValidEndDate || isStartBeforeEnd) {
        fieldDateOrderErrorSetter(undefined);
        otherDateOrderFieldSetter(undefined);
        return isValidStartDate && isValidEndDate; // form is valid, both dates defined, in correct order
    }
    const isCausedByTime = isSameDay(start, end);
    fieldDateOrderErrorSetter({
        isDateOrderError: !isCausedByTime,
        isTimeOrderError: isCausedByTime,
    });
    otherDateOrderFieldSetter(undefined);
    return false; // form is invalid at this point
};
// if property was set on the object, even if it was set to undefined, we want return the value,
// otherwise we want to return default value from function parameter
function getValueOrDefault(state, valueKey, defaultValue) {
    return Object.prototype.hasOwnProperty.call(state, valueKey) ? state[valueKey] : defaultValue;
}
const useRangeState = (range, onRangeChange, closeCalendarPopup, selectedInput, submitForm) => {
    const [startDate, setStartDate] = useState(range.from);
    const [startTime, setStartTime] = useState(getTimeFromDate(range.from));
    const [endDate, setEndDate] = useState(range.to);
    const [endTime, setEndTime] = useState(getTimeFromDate(range.to));
    const [startDateTimeErrors, setStartDateTimeErrors] = useState();
    const [endDateTimeErrors, setEndDateTimeErrors] = useState();
    const validateStartDate = useCallback((startDate, endDate) => setStartAfterEndDateError(setStartDateTimeErrors, setEndDateTimeErrors, startDate, endDate), []);
    const validateEndDate = useCallback((startDate, endDate) => setStartAfterEndDateError(setEndDateTimeErrors, setStartDateTimeErrors, startDate, endDate), []);
    const updateRangeState = useCallback((newState, dateValidator, shouldSubmitForm) => {
        const adjustedStartDate = setTimeToDate(getValueOrDefault(newState, "startDate", startDate), getValueOrDefault(newState, "startTime", startTime));
        const adjustedEndDate = setTimeToDate(getValueOrDefault(newState, "endDate", endDate), getValueOrDefault(newState, "endTime", endTime));
        onRangeChange({ from: adjustedStartDate, to: adjustedEndDate });
        const isFormValid = dateValidator(adjustedStartDate, adjustedEndDate);
        // submit form when user pressed Enter and form is valid
        if (isFormValid && !!shouldSubmitForm) {
            // deffer submit to the next render loop, newest values are not propagated to state yet
            setTimeout(submitForm, 0);
        }
    }, [onRangeChange, startDate, startTime, endDate, endTime, submitForm]);
    const onStartDateChange = useCallback((date, shouldSubmitForm) => {
        setStartDate(date);
        updateRangeState({ startDate: date }, validateStartDate, shouldSubmitForm);
    }, [updateRangeState, validateStartDate]);
    const onEndDateChange = useCallback((date, shouldSubmitForm) => {
        setEndDate(date);
        updateRangeState({ endDate: date }, validateEndDate, shouldSubmitForm);
    }, [updateRangeState, validateEndDate]);
    const onStartTimeChange = useCallback((time, shouldSubmitForm) => {
        setStartTime(time);
        updateRangeState({ startTime: time }, validateStartDate, shouldSubmitForm);
    }, [updateRangeState, validateStartDate]);
    const onEndTimeChange = useCallback((time, shouldSubmitForm) => {
        setEndTime(time);
        updateRangeState({ endTime: time }, validateEndDate, shouldSubmitForm);
    }, [updateRangeState, validateEndDate]);
    const onCalendarDateSelect = useCallback((_range, selectedDate) => {
        if (selectedInput === "from") {
            onStartDateChange(selectedDate);
        }
        else {
            onEndDateChange(selectedDate);
        }
        closeCalendarPopup();
    }, [selectedInput, closeCalendarPopup, onStartDateChange, onEndDateChange]);
    return {
        startDate,
        startTime,
        endDate,
        endTime,
        startDateTimeErrors,
        endDateTimeErrors,
        onCalendarDateSelect,
        onStartDateChange,
        onStartTimeChange,
        onEndDateChange,
        onEndTimeChange,
    };
};
const DateRangePickerComponent = ({ range, onRangeChange, dateFormat, dayPickerProps, intl, isMobile, isTimeEnabled, weekStart = "Sunday", shouldOverlayDatePicker = false, submitForm, }) => {
    const { selectedInput, isCalendarPopupOpen, onDateInputKeyDown, onStartDateClick, onEndDateClick, calendarPopupRef, startDateInputRef, endDateInputRef, closeCalendarPopup, } = useCalendarPopup();
    const { startDate, startTime, endDate, endTime, startDateTimeErrors, endDateTimeErrors, onCalendarDateSelect, onStartDateChange, onStartTimeChange, onEndDateChange, onEndTimeChange, } = useRangeState(range, onRangeChange, closeCalendarPopup, selectedInput, submitForm);
    const StartDateField = (React.createElement(StartDateInputField, { ref: startDateInputRef, date: startDate, time: startTime, onDateChange: onStartDateChange, onTimeChange: onStartTimeChange, onInputClick: onStartDateClick, onDateInputKeyDown: onDateInputKeyDown, dateFormat: dateFormat, isMobile: isMobile, isTimeEnabled: isTimeEnabled, errors: startDateTimeErrors, intl: intl }));
    const EndDateField = (React.createElement(EndDateInputField, { ref: endDateInputRef, date: endDate, time: endTime, onDateChange: onEndDateChange, onTimeChange: onEndTimeChange, onInputClick: onEndDateClick, onDateInputKeyDown: onDateInputKeyDown, dateFormat: dateFormat, isMobile: isMobile, isTimeEnabled: isTimeEnabled, errors: endDateTimeErrors, intl: intl }));
    const datePickerRange = { from: startDate, to: endDate };
    const DatePicker = (React.createElement(DayPicker, { ref: calendarPopupRef, mode: selectedInput, originalDateRange: datePickerRange, selectedDateRange: datePickerRange, onDateRangeSelect: onCalendarDateSelect, alignTo: `.gd-date-range-picker-${isTimeEnabled ? selectedInput : "from"}`, calendarClassNames: `gd-date-range-picker-picker s-date-range-calendar-${selectedInput}`, dayPickerProps: dayPickerProps, weekStart: weekStart, renderAsOverlay: shouldOverlayDatePicker, intl: intl }));
    const HintPanel = isMobile ? null : (React.createElement(DateRangeHint, { dateFormat: dateFormat, isTimeEnabled: isTimeEnabled, dateHintId: DATE_INPUT_HINT_ID, timeHintId: TIME_INPUT_HINT_ID, intl: intl }));
    if (isTimeEnabled) {
        const isFromInputDatePickerOpen = selectedInput === "from" && isCalendarPopupOpen;
        const isToInputDatePickerOpen = selectedInput === "to" && isCalendarPopupOpen;
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "gd-date-range-picker datetime s-date-range-picker" },
                StartDateField,
                isFromInputDatePickerOpen ? DatePicker : null,
                EndDateField,
                isToInputDatePickerOpen ? DatePicker : null),
            HintPanel));
    }
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "gd-date-range-picker gd-date-range-row s-date-range-picker" },
            StartDateField,
            EndDateField),
        isCalendarPopupOpen ? DatePicker : null,
        HintPanel));
};
export const DateRangePicker = injectIntl(DateRangePickerComponent);
//# sourceMappingURL=DateRangePicker.js.map