// (C) 2019-2025 GoodData Corporation
import React, { useState, useCallback, useMemo } from "react";
import cx from "classnames";
import { RelativeRangeDynamicSelect, } from "../DynamicSelect/RelativeRangeDynamicSelect.js";
import { findRelativeDateFilterOptionByLabel, findRelativeDateFilterOptionByValue, getRelativeDateFilterItems, } from "../DynamicSelect/utils.js";
import { injectIntl } from "react-intl";
import { defaultVisibleItemsRange } from "../Select/VirtualizedSelectMenu.js";
import { useIdPrefixed } from "@gooddata/sdk-ui-kit";
import { itemToString } from "../Select/utils.js";
var RelativeRangePickerErrorType;
(function (RelativeRangePickerErrorType) {
    RelativeRangePickerErrorType["INVALID_VALUE"] = "INVALID_VALUE";
    RelativeRangePickerErrorType["EMPTY_VALUE"] = "EMPTY_VALUE";
})(RelativeRangePickerErrorType || (RelativeRangePickerErrorType = {}));
const SelectWrapper = (props) => {
    return (React.createElement("div", { className: cx(props.className, "gd-relative-range-picker-select-wrapper s-relative-range-picker-select-wrapper") },
        React.createElement("label", { className: "gd-label", id: props.labelId }, props.label),
        props.children,
        props.errorMessage ? (React.createElement("div", { id: props.errorId, className: "s-gd-relative-range-picker-error gd-relative-range-picker-select-wrapper__error-message" }, props.errorMessage)) : null));
};
const getItemsFactory = (granularity, isMobile, intl) => {
    return (value) => {
        const items = getRelativeDateFilterItems(value, granularity, intl);
        // separators are not needed in mobile as all the items have borders
        return isMobile ? items.filter((item) => item.type !== "separator") : items;
    };
};
const getInputValueFromValue = (value, items) => {
    const selectedItem = value !== undefined ? findRelativeDateFilterOptionByValue(items, value) : null;
    return selectedItem ? itemToString(selectedItem) : value ? value.toString() : "";
};
const RelativeRangePickerSelect = React.memo((props) => {
    const { label, value, inputValue, error, onChange, onInputChange, onBlur, commonProps, intl, isMobile, className, wrapperClassName, } = props;
    const labelId = useIdPrefixed("label");
    const errorId = useIdPrefixed("error");
    return (React.createElement(SelectWrapper, { label: label, labelId: labelId, errorMessage: error, errorId: errorId, className: wrapperClassName },
        React.createElement(RelativeRangeDynamicSelect, { ...commonProps, value: value, onChange: onChange, inputValue: inputValue, onInputValueChange: onInputChange, onBlur: onBlur, placeholder: intl.formatMessage({ id: "filters.relative.placeholder" }), className: cx(className, "gd-relative-range-picker-picker", {
                "gd-relative-range-picker-picker-mobile": isMobile,
                "has-error": error !== null,
            }), accessibilityConfig: {
                labelId,
                ...(error && { descriptionId: errorId }),
            } })));
});
const mobileVisibleItemsRange = 5;
const RelativeRangePickerComponent = (props) => {
    const { selectedFilterOption, intl, isMobile, accessibilityConfig, id, onSelectedFilterOptionChange } = props;
    const getItems = useMemo(() => getItemsFactory(selectedFilterOption.granularity, isMobile, intl), [selectedFilterOption.granularity, isMobile, intl]);
    const [fromError, setFromError] = useState(null);
    const [toError, setToError] = useState(null);
    const [fromInputValue, setFromInputValue] = useState(getInputValueFromValue(selectedFilterOption.from, selectedFilterOption.from !== undefined ? getItems(selectedFilterOption.from.toString()) : []));
    const [toInputValue, setToInputValue] = useState(getInputValueFromValue(selectedFilterOption.to, selectedFilterOption.to !== undefined ? getItems(selectedFilterOption.to.toString()) : []));
    const commonProps = useMemo(() => ({
        visibleItemsRange: isMobile ? mobileVisibleItemsRange : defaultVisibleItemsRange,
        optionClassName: "s-relative-date-filter-option s-do-not-close-dropdown-on-click",
        getItems,
        inputValue: "",
        onInputValueChange: () => { },
    }), [isMobile, getItems]);
    const handleFromChange = useCallback((from) => {
        onSelectedFilterOptionChange({ ...selectedFilterOption, from });
        if (from !== undefined) {
            setFromInputValue(getInputValueFromValue(from, getItems(from?.toString())));
            setFromError(null);
        }
    }, [selectedFilterOption, onSelectedFilterOptionChange, getItems]);
    const handleToChange = useCallback((to) => {
        onSelectedFilterOptionChange({ ...selectedFilterOption, to });
        if (to !== undefined) {
            setToInputValue(getInputValueFromValue(to, getItems(to?.toString())));
            setToError(null);
        }
    }, [selectedFilterOption, onSelectedFilterOptionChange, getItems]);
    const validator = useCallback((inputValue) => {
        if (inputValue === "") {
            return RelativeRangePickerErrorType.EMPTY_VALUE;
        }
        const items = getItems(inputValue);
        const matchingItem = findRelativeDateFilterOptionByLabel(items, inputValue);
        if (matchingItem) {
            return null;
        }
        return RelativeRangePickerErrorType.INVALID_VALUE;
    }, [getItems]);
    const handleFromInputChange = useCallback((value) => {
        setFromInputValue(value);
        // During typing, we only validate but don't show errors
        const validationResult = validator(value);
        if (validationResult) {
            // Store the error but don't show it yet
            handleFromChange(undefined);
        }
        else {
            setFromError(null);
            const items = getItems(value);
            const matchingItem = findRelativeDateFilterOptionByLabel(items, value);
            if (matchingItem) {
                handleFromChange(matchingItem.value);
            }
        }
    }, [validator, handleFromChange, getItems]);
    const handleToInputChange = useCallback((value) => {
        setToInputValue(value);
        // During typing, we only validate but don't show errors
        const validationResult = validator(value);
        if (validationResult) {
            // Store the error but don't show it yet
            handleToChange(undefined);
        }
        else {
            setToError(null);
            const items = getItems(value);
            const matchingItem = findRelativeDateFilterOptionByLabel(items, value);
            if (matchingItem) {
                handleToChange(matchingItem.value);
            }
        }
    }, [validator, handleToChange, getItems]);
    const handleFromBlur = useCallback(() => {
        const validationResult = validator(fromInputValue);
        if (validationResult) {
            switch (validationResult) {
                case RelativeRangePickerErrorType.INVALID_VALUE:
                    setFromError(intl.formatMessage({ id: "filters.relative.from.invalid.value" }));
                    break;
                case RelativeRangePickerErrorType.EMPTY_VALUE:
                    setFromError(intl.formatMessage({ id: "filters.relative.from.empty.value" }));
                    break;
            }
            handleFromChange(undefined);
        }
        else {
            setFromError(null);
        }
    }, [validator, fromInputValue, intl, handleFromChange]);
    const handleToBlur = useCallback(() => {
        const validationResult = validator(toInputValue);
        if (validationResult) {
            switch (validationResult) {
                case RelativeRangePickerErrorType.INVALID_VALUE:
                    setToError(intl.formatMessage({ id: "filters.relative.to.invalid.value" }));
                    break;
                case RelativeRangePickerErrorType.EMPTY_VALUE:
                    setToError(intl.formatMessage({ id: "filters.relative.to.empty.value" }));
                    break;
            }
            handleToChange(undefined);
        }
        else {
            setToError(null);
        }
    }, [validator, toInputValue, intl, handleToChange]);
    return (React.createElement("div", { id: id, className: "gd-relative-range-picker s-relative-range-picker", role: accessibilityConfig?.role, "aria-labelledby": accessibilityConfig?.ariaLabelledBy },
        React.createElement(RelativeRangePickerSelect, { className: "s-relative-range-picker-from", wrapperClassName: "s-relative-range-picker-from-wrapper", label: intl.formatMessage({ id: "filters.relative.from.label" }), value: selectedFilterOption.from, inputValue: fromInputValue, error: fromError, onChange: handleFromChange, onInputChange: handleFromInputChange, onBlur: handleFromBlur, commonProps: commonProps, intl: intl, isMobile: isMobile }),
        React.createElement(RelativeRangePickerSelect, { className: "s-relative-range-picker-to", wrapperClassName: "s-relative-range-picker-to-wrapper", label: intl.formatMessage({ id: "filters.relative.to.label" }), value: selectedFilterOption.to, inputValue: toInputValue, error: toError, onChange: handleToChange, onInputChange: handleToInputChange, onBlur: handleToBlur, commonProps: commonProps, intl: intl, isMobile: isMobile })));
};
export const RelativeRangePicker = injectIntl(RelativeRangePickerComponent);
//# sourceMappingURL=RelativeRangePicker.js.map